﻿-- locals, helperfunctions
local string_gsub, string_format = string.gsub, string.format
local table_sort, table_concat = table.sort, table.concat

-- LibDataBroker
local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
local LDBIcon = ldb and LibStub("LibDBIcon-1.0",true)
local LibQTip = LibStub('LibQTip-1.0')
local Ailo = LibStub("AceAddon-3.0"):NewAddon("Ailo")
local L = LibStub("AceLocale-3.0"):GetLocale("Ailo", false)

Ailo.frame = CreateFrame("Frame")
local function onEventFunc(self, event, ...)
    if event == "UPDATE_INSTANCE_INFO" then
        self:UpdateRaidIDDB()
    end
end
Ailo.frame:RegisterEvent("UPDATE_INSTANCE_INFO")
Ailo.frame:SetScript("OnEvent", onEventFunc)

local function setColor(info, r, g, b, a)
    Ailo.db.profile[info[#info]] = { r = r, g = g, b = b, a = a }
end

local function getColor(info)
    return Ailo.db.profile[info[#info]].r, Ailo.db.profile[info[#info]].g, Ailo.db.profile[info[#info]].b, Ailo.db.profile[info[#info]].a
end

local options = {
    name = "Ailo",
    type = 'group',
    args = {
        genconfig = {
            name = L["General Settings"],
            type = 'group',
            order = 1,
            args = {
                savedraid = {
                    name = L["Saved raid color"],
                    desc = L["SAVED_RAID_DESC"],
                    type = 'color',
                    order = 1,
                    get  = getColor,
                    set  = setColor,
                    hasAlpha = true,
                },
                freeraid = {
                    name = L["Free raid color"],
                    desc = L["FREE_RAID_DESC"],
                    type = 'color',
                    order = 2,
                    get  = getColor,
                    set  = setColor,
                    hasAlpha = true,
                },
                minimapIcon = {
                    type = "toggle",
                    name = L["Show minimap button"],
                    desc = L["Show the Ailo minimap button"],
                    order = 3,
                    get = function(info) return not Ailo.db.profile.minimapIcon.hide end,
                    set = function(info, value)
                        Ailo.db.profile.minimapIcon.hide = not value
                        if value then LDBIcon:Show("Ailo") else LDBIcon:Hide("Ailo") end
                    end,
                },
            },
        },
    },
}

local defaults = {
    profile = {
        savedraid = { r=1, g=0, b=0, a=1 },
        freeraid  = { r=0, g=1, b=0, a=1 },
        minimapIcon = {
            hide = false,
            minimapPos = 220,
            radius = 80,
        },
    },
    global = {
        chars = {},
        raids = {},
        nextPurge = 0,
    },
}


function Ailo:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("AiloDB", defaults)

    local AiloLDB = ldb:NewDataObject("Ailo", {
        type = "data source",
        text = "Ailo",
        icon = "Interface\\Icons\\Achievement_Dungeon_UlduarRaid_Archway_01.png",
        OnClick = function(clickedframe, button)
            if button == "RightButton" then 
                InterfaceOptionsFrame_OpenToCategory(Ailo.optionsFrame) 
            else 
                Ailo:ManualPlayerUpdate() 
            end
        end,
        OnEnter = function(tt)
            local tooltip = LibQTip:Acquire("AiloTooltip", 1, "LEFT") 
            Ailo.tooltip = tooltip
            Ailo:PrepareTooltip(tooltip) 
            tooltip:SmartAnchorTo(tt)
            tooltip:Show()
        end,
        OnLeave = function(tt)
            LibQTip:Release(Ailo.tooltip)
            Ailo.tooltip = nil
        end,
    })

    options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
    LDBIcon:Register("Ailo", AiloLDB, self.db.profile.minimapIcon)
end

Ailo.frame.InstanceNameFormatString = "%s%d%s"
local raidorder= {}
function Ailo:PrepareTooltip(tooltip)
    --[[[ Cell are just colored green/red
               ToC       VoA
            10     25   10  25
          hc nhc hc nhc nhc nhc
    Char1 [] [ ] [] [ ] [ ] [ ]
    Char2 [] [ ] [] [ ] [ ] [ ]
    Char3 [] [ ] [] [ ] [ ] [ ]
    ]]--
    local charsdb = self.db.global.chars
    local raidsdb = self.db.global.raids
    
    if next(charsdb) then
        -- At least one char is saved to some 
        tooltip:AddHeader(L["Raid"]) -- Raid
        tooltip:AddHeader(L["Size"]) -- Size
        tooltip:AddHeader(L["Diff"]) -- Heroic
        local nextPurge = self.db.global.nextPurge
        if nextPurge > 0 and time() > nextPurge then
            -- Only search 
            self:PurgeOldRaidIDs()
            self:TrimRaidTable()
        end
    
        local raid, size, difficulties,  difficulty, colcount
        local colcount, numdifficulties
        for raid, sizes in pairs(raidsdb) do
            colcount = 0 -- Span needed for the 'Raid' cell above the 'Size' cells
    
            for size, difficulties in pairs(sizes) do
                numdifficulties = 0 -- Span needed for the 'Size' cell above the 'Difficulty' cells
    
                for difficulty, _ in pairs(difficulties) do
                    colcount = colcount +1
                    numdifficulties = numdifficulties +1
    
                    lastcolumn = tooltip:AddColumn("CENTER")
                    tooltip:SetCell(3, lastcolumn, difficulty)
    
                    raidorder[(string_format(self.frame.InstanceNameFormatString, raid, size, difficulty))] = lastcolumn
    
                end
                tooltip:SetCell(2, lastcolumn - numdifficulties+1, size, numdifficulties)
            end
            tooltip:SetCell(1, lastcolumn - colcount+1, raid, colcount)
        end
        
        local currentPlayer, raids, currentRaid, lastline
        for currentPlayer, raids in pairs(charsdb) do 
            lastline = tooltip:AddLine(currentPlayer)
            for i = tooltip:GetColumnCount(),2,-1 do
                tooltip:SetCell(lastline, i, "")
                
                tooltip:SetCellColor(lastline, i, self.db.profile.freeraid.r, self.db.profile.freeraid.g, self.db.profile.freeraid.b, self.db.profile.freeraid.a)
            end
            for currentRaid, _ in pairs(raids) do
                tooltip:SetCellColor(lastline, raidorder[currentRaid], self.db.profile.savedraid.r, self.db.profile.savedraid.g, self.db.profile.savedraid.b, self.db.profile.savedraid.a)
            end
        end
    else
        -- No saved raids at all
        tooltip:AddHeader(L["No saved raids found"])
    end
end

function Ailo:PurgeOldRaidIDs()
    local charsdb = self.db.global.chars
    local currentPlayer, raids, currentRaid, expireTime
    local now = time()
    for currentPlayer, raids in pairs(charsdb) do 
        for currentRaid, expireTime in pairs(raids) do
            if now > expireTime then
                self.db.global.chars[currentPlayer][currentRaid] = nil
            end
        end
        if not next(charsdb[currentPlayer]) then 
            self.db.global.chars[currentPlayer] = nil
        end
    end
end

function Ailo:ExtendRaidTable(name, size, difficulty)
    self.db.global.raids[name] = self.db.global.raids[name] or {}
    self.db.global.raids[name][size] =  self.db.global.raids[name][size] or {}
    self.db.global.raids[name][size][difficulty] = true
end

function Ailo:TrimRaidTable()
    local charsdb = self.db.global.chars
    local currentPlayer, raids, currentRaid
    local raidsdb = self.db.global.raids
    local raid, size, difficulties,  difficulty
    local isUsed, raidString = false
    for raid, sizes in pairs(raidsdb) do
        for size, difficulties in pairs(sizes) do
            for difficulty, _ in pairs(difficulties) do
                isUsed = false
                raidString = string_format(self.frame.InstanceNameFormatString, raid, size, difficulty)
                for currentPlayer, raids in pairs(charsdb) do 
                    if raids and raids[raidString] then
                        isUsed = true
                        break
                    end
                end
                if not isUsed then
                    self:DeleteFromRaidTable(raid, size, difficulty)
                end
            end
        end
    end
end

function Ailo:DeleteFromRaidTable(name, size, difficulty)
    self.db.global.raids[name][size][difficulty] = nil
    if not next(self.db.global.raids[name][size]) then
        self.db.global.raids[name][size] = nil
    end
    if not next(self.db.global.raids[name]) then
        self.db.global.raids[name] = nil
    end
end

function Ailo:ManualPlayerUpdate()
    local thisplayer, currentRaid = UnitName("player")
    if not self.db.global.chars[thisplayer] then 
        self.db.global.chars[thisplayer] = {}
    else 
        for currentRaid, _ in pairs(self.db.global.chars[thisplayer]) do
            self.db.global.chars[thisplayer][currentRaid] = nil
        end
    end
    self:PurgeOldRaidIDs()
    self:TrimRaidTable()
    self.frame:UpdateRaidIDDB()
end

function Ailo:SaveRaidForChar(char, instance, expireTime)
    if not self.db.global.chars[char] then
        self.db.global.chars[char] = {}
    end
    self.db.global.chars[char][instance] = expireTime
    
    if expireTime < self.db.global.nextPurge or self.db.global.nextPurge == 0 then
        self.db.global.nextPurge = expireTime
    end
end

function Ailo.frame:UpdateRaidIDDB()
    if 80 > UnitLevel("player") then return end
    local now, thisplayer, index = time(), UnitName("player")
    local instanceName, instanceReset, instanceDifficulty, locked, isRaid, maxPlayers
    local instanceNameAbbr, difficulty, instanceString
    for index=1,GetNumSavedInstances() do
        --instanceName, instanceID, instanceReset, instanceDifficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName = GetSavedInstanceInfo(index)
        instanceName, _, instanceReset, instanceDifficulty, locked, _, _, isRaid, maxPlayers, _ = GetSavedInstanceInfo(index)
        if locked and isRaid then
            instanceNameAbbr = string_gsub(instanceName, "(%a)[%l%p]*[%s%-]*", "%1")
            difficulty       = instanceDifficulty > 2 and "hc" or "nhc"
            instanceString   = string_format(self.InstanceNameFormatString, instanceNameAbbr, maxPlayers, difficulty)
            Ailo:ExtendRaidTable(instanceNameAbbr, maxPlayers, difficulty)
            Ailo:SaveRaidForChar(thisplayer, instanceString, now+instanceReset)
        end
    end
end

Ailo.options = options    
LibStub("AceConfig-3.0"):RegisterOptionsTable("Ailo", options)
Ailo.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Ailo")